"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validateUniqueItems = exports.validateItemsAgainstSchema = exports.validateItems = exports.validateContains = exports.arrayValidator = exports.ERROR_NOT_FOUND_CONTAINS = exports.ERROR_MIN_CONTAINS = exports.ERROR_MAX_CONTAINS = exports.ERROR_DUPLICATE_ITEMS = exports.ERROR_ADDITIONAL_ITEMS = void 0;
var _toPointer = require("@ui-schema/json-pointer/toPointer");
var _ValidatorOutput = require("@ui-schema/ui-schema/ValidatorOutput");
var _immutable = require("immutable");
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
function _createForOfIteratorHelper(r, e) { var t = "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"]; if (!t) { if (Array.isArray(r) || (t = _unsupportedIterableToArray(r)) || e && r && "number" == typeof r.length) { t && (r = t); var _n = 0, F = function F() {}; return { s: F, n: function n() { return _n >= r.length ? { done: !0 } : { done: !1, value: r[_n++] }; }, e: function e(r) { throw r; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var o, a = !0, u = !1; return { s: function s() { t = t.call(r); }, n: function n() { var r = t.next(); return a = r.done, r; }, e: function e(r) { u = !0, o = r; }, f: function f() { try { a || null == t["return"] || t["return"](); } finally { if (u) throw o; } } }; }
function _toConsumableArray(r) { return _arrayWithoutHoles(r) || _iterableToArray(r) || _unsupportedIterableToArray(r) || _nonIterableSpread(); }
function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _iterableToArray(r) { if ("undefined" != typeof Symbol && null != r[Symbol.iterator] || null != r["@@iterator"]) return Array.from(r); }
function _arrayWithoutHoles(r) { if (Array.isArray(r)) return _arrayLikeToArray(r); }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
var ERROR_DUPLICATE_ITEMS = exports.ERROR_DUPLICATE_ITEMS = 'duplicate-items';
var ERROR_NOT_FOUND_CONTAINS = exports.ERROR_NOT_FOUND_CONTAINS = 'not-found-contains';
var ERROR_MIN_CONTAINS = exports.ERROR_MIN_CONTAINS = 'min-contains';
var ERROR_MAX_CONTAINS = exports.ERROR_MAX_CONTAINS = 'max-contains';
var ERROR_ADDITIONAL_ITEMS = exports.ERROR_ADDITIONAL_ITEMS = 'additional-items';
var findDuplicates = function findDuplicates(arr, isEqual) {
  var arrInterop = Array.isArray(arr) ? {
    size: arr.length,
    get: function get(i) {
      return arr[i];
    }
  } : arr;
  var seen = [];
  var _loop = function _loop() {
    var val = arrInterop.get(i);
    var existing = seen.find(function (entry) {
      return isEqual(entry.value, val);
    });
    if (existing) {
      existing.indexes.push(i);
    } else {
      seen.push({
        value: val,
        indexes: [i]
      });
    }
  };
  for (var i = 0; i < arrInterop.size; i++) {
    _loop();
  }
  return seen.filter(function (entry) {
    return entry.indexes.length > 1;
  });
};
var validateUniqueItems = exports.validateUniqueItems = function validateUniqueItems(schema, value, params) {
  var uniqueItems = schema.get('uniqueItems');
  if (!uniqueItems) {
    return true;
  }
  var duplicates = findDuplicates(value, _immutable.is);
  if (duplicates.length > 0 && params) {
    params.output.addError({
      error: ERROR_DUPLICATE_ITEMS,
      keywordLocation: (0, _toPointer.toPointer)([].concat(_toConsumableArray(params.keywordLocation), ['uniqueItems'])),
      instanceLocation: (0, _toPointer.toPointer)(params.instanceLocation),
      context: {
        duplicates: duplicates.map(function (d) {
          return {
            value: d.value,
            indexes: d.indexes
          };
        })
      }
    });
  }
  return duplicates.length === 0;
};
var validateItemsAgainstSchema = exports.validateItemsAgainstSchema = function validateItemsAgainstSchema(itemsToValidate, schemaToValidate, params) {
  var found = 0;
  var i = 0;
  var _iterator = _createForOfIteratorHelper(itemsToValidate),
    _step;
  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var val = _step.value;
      var result = params.validate(schemaToValidate, val, _objectSpread(_objectSpread({}, params), {}, {
        instanceLocation: [].concat(_toConsumableArray(params.instanceLocation), [i]),
        instanceKey: i,
        output: new _ValidatorOutput.ValidatorOutput(),
        context: {}
      }));
      if (result.valid) {
        found++;
      }
      i++;
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }
  return {
    found: found
  };
};
var normalizeArraySchema = function normalizeArraySchema(schema) {
  var itemsTmp = schema.get('items');
  var prefixItemsTmp = schema.get('prefixItems');
  var additionalItemsTmp = schema.get('additionalItems');
  var prefixItems = undefined;
  var items = undefined;
  var additionalItemsAllowed = true;
  if (prefixItemsTmp !== undefined && _immutable.List.isList(prefixItemsTmp)) {
    prefixItems = prefixItemsTmp;
    if (itemsTmp === false) {
      additionalItemsAllowed = false;
    } else if (_typeof(itemsTmp) === 'object' && itemsTmp !== null) {
      items = itemsTmp;
    }
  } else if (_immutable.List.isList(itemsTmp)) {
    prefixItems = itemsTmp;
    if (additionalItemsTmp === false) {
      additionalItemsAllowed = false;
    } else if (_typeof(additionalItemsTmp) === 'object' && additionalItemsTmp !== null) {
      items = additionalItemsTmp;
    }
  } else if (_typeof(itemsTmp) === 'object' && itemsTmp !== null) {
    items = itemsTmp;
  } else if (itemsTmp === false) {
    additionalItemsAllowed = false;
  }
  return {
    prefixItems: prefixItems,
    items: items,
    additionalItemsAllowed: additionalItemsAllowed
  };
};
var validateItems = exports.validateItems = function validateItems(schema, value, params) {
  var _normalizeArraySchema = normalizeArraySchema(schema),
    prefixItems = _normalizeArraySchema.prefixItems,
    items = _normalizeArraySchema.items,
    additionalItemsAllowed = _normalizeArraySchema.additionalItemsAllowed;
  var prefixItemsCount = (prefixItems === null || prefixItems === void 0 ? void 0 : prefixItems.size) || 0;
  var i = 0;
  var _iterator2 = _createForOfIteratorHelper(value),
    _step2;
  try {
    for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
      var val = _step2.value;
      var itemSchema = undefined;
      var keyword = void 0;
      if (i < prefixItemsCount) {
        itemSchema = prefixItems.get(i);
        keyword = schema.has('prefixItems') ? 'prefixItems' : 'items';
      } else {
        if (!additionalItemsAllowed) {
          params.output.addError({
            error: ERROR_ADDITIONAL_ITEMS,
            keywordLocation: (0, _toPointer.toPointer)([].concat(_toConsumableArray(params.keywordLocation), [schema.get('items') === false ? 'items' : 'additionalItems'])),
            instanceLocation: (0, _toPointer.toPointer)([].concat(_toConsumableArray(params.instanceLocation), [i]))
          });
          i++;
          continue;
        }
        itemSchema = items;
        keyword = 'items';
      }
      if (itemSchema) {
        params.validate(itemSchema, val, _objectSpread(_objectSpread({}, params), {}, {
          keywordLocation: prefixItems !== null && prefixItems !== void 0 && prefixItems.has(i) ? [].concat(_toConsumableArray(params.keywordLocation), [keyword, i]) : [].concat(_toConsumableArray(params.keywordLocation), [keyword]),
          instanceLocation: [].concat(_toConsumableArray(params.instanceLocation), [i]),
          instanceKey: i
        }));
      }
      i++;
    }
  } catch (err) {
    _iterator2.e(err);
  } finally {
    _iterator2.f();
  }
};
var validateContains = exports.validateContains = function validateContains(schema, value, params) {
  var _ref;
  var containsSchema = schema.get('contains');
  if (_typeof(containsSchema) !== 'object' || containsSchema === null) return;
  var minContains = (_ref = schema.get('minContains')) !== null && _ref !== void 0 ? _ref : 1;
  var maxContains = schema.get('maxContains');
  var _validateItemsAgainst = validateItemsAgainstSchema(value, containsSchema, params),
    found = _validateItemsAgainst.found;
  if (found < minContains) {
    var isDefaultMinContains = minContains === 1 && !schema.has('minContains');
    params.output.addError({
      error: isDefaultMinContains ? ERROR_NOT_FOUND_CONTAINS : ERROR_MIN_CONTAINS,
      keywordLocation: (0, _toPointer.toPointer)([].concat(_toConsumableArray(params.keywordLocation), [isDefaultMinContains ? 'contains' : 'minContains'])),
      instanceLocation: (0, _toPointer.toPointer)(params.instanceLocation),
      context: {
        minContains: minContains,
        found: found
      }
    });
  }
  if (typeof maxContains === 'number' && found > maxContains) {
    params.output.addError({
      error: ERROR_MAX_CONTAINS,
      keywordLocation: (0, _toPointer.toPointer)([].concat(_toConsumableArray(params.keywordLocation), ['maxContains'])),
      instanceLocation: (0, _toPointer.toPointer)(params.instanceLocation),
      context: {
        maxContains: maxContains,
        found: found
      }
    });
  }
};
var arrayValidator = exports.arrayValidator = {
  types: ['array'],
  validate: function validate(schema, value, params) {
    if (!(_immutable.List.isList(value) || Array.isArray(value))) {
      return;
    }
    if (schema.get('uniqueItems')) {
      validateUniqueItems(schema, value, params);
    }
    if (schema.has('contains')) {
      validateContains(schema, value, params);
    }
    if (params.recursive) {
      validateItems(schema, value, params);
    }
  }
};