import { ValidatorParams, ValidatorState, ValidatorHandler } from '@ui-schema/json-schema/Validator';
import { List } from 'immutable';
import { UISchemaMap } from '@ui-schema/json-schema/Definitions';
export declare const ERROR_DUPLICATE_ITEMS = "duplicate-items";
export declare const ERROR_NOT_FOUND_CONTAINS = "not-found-contains";
export declare const ERROR_MIN_CONTAINS = "min-contains";
export declare const ERROR_MAX_CONTAINS = "max-contains";
export declare const ERROR_ADDITIONAL_ITEMS = "additional-items";
/**
 * Validates if all items in an array are unique.
 */
export declare const validateUniqueItems: (schema: UISchemaMap, value: List<any> | any[], params?: ValidatorParams & ValidatorState) => boolean;
/**
 * A helper to validate a list of items against a single schema.
 * Used for `contains` validation. It counts the number of valid items
 * without polluting the main output with errors from non-matching items.
 */
export declare const validateItemsAgainstSchema: (itemsToValidate: List<any> | any[], schemaToValidate: UISchemaMap, params: ValidatorParams & ValidatorState) => {
    found: number;
};
/**
 * Validates array items against `items`, `prefixItems`, and `additionalItems` keywords.
 * It handles both tuple and list validation across different JSON Schema drafts.
 */
export declare const validateItems: (schema: UISchemaMap, value: List<any> | any[], params: ValidatorParams & ValidatorState) => void;
/**
 * Validates the array against `contains`, `minContains`, and `maxContains` keywords.
 */
export declare const validateContains: (schema: UISchemaMap, value: List<any> | any[], params: ValidatorParams & ValidatorState) => void;
/**
 * The main validator for the `array` type.
 * Handles `uniqueItems`, `contains`, and recursively validates `items`/`prefixItems`.
 */
export declare const arrayValidator: ValidatorHandler;
